@@ -1,5 +1,5 @@
 package Devel::Camelcadedb;
-our $VERSION = "1.6.1.5";
+our $VERSION = "1.6.1.6";
 
 # http://perldoc.perl.org/DB.html
 # http://perldoc.perl.org/perldebug.html
@@ -113,8 +113,9 @@ my $glob_slots = join '|', @glob_slots;
 my $_dev_mode = 0;                              # enable this to get verbose STDERR output from process
 my $_debug_log_fh = *STDERR;                    # debug log fh. If omited, file will be created
 my $_debug_log_filename = 'current_debug.log';
-my $_debug_sub_handler = 0;                     # debug entering/leaving subs, works in dev mode
+my $_debug_sub_handler = 1;                     # debug entering/leaving subs, works in dev mode
 my $_debug_load_handler = 0;                    # debug modules loading
+my $_debug_breakpoints = 0;                     # debug breakpoints setting
 
 my $_script_charset = 'utf8';   # all sources and strings without utf flag will be encoded from this encoding to the utf
 
@@ -921,14 +922,15 @@ sub _event_handler
 
 sub _enter_frame
 {
-    my ($old_db_single) = @_;
+    my ($old_db_single, $wantarray) = @_;
 
-    _report "Entering frame %s: %s %s-%s-%s",
+    _report "Entering frame %s: %s %s-%s-%s, %s",
         scalar @$_stack_frames + 1,
         $DB::sub,
         $DB::trace // 'undef',
         $DB::signal // 'undef',
         $old_db_single // 'undef',
+            $wantarray ? 'ARRAY' : defined $wantarray ? 'SCALAR' : 'VOID'
         if $_debug_sub_handler && $_dev_mode;
 
     $frame_prefix = $frame_prefix_step x (scalar @$_stack_frames + 1);
@@ -1012,7 +1014,7 @@ sub _get_loaded_breakpoints_by_real_path
 
     if ($_loaded_breakpoints{$real_path})
     {
-        _report "Found real breakpoints" if $_dev_mode;
+        _report "Found real breakpoints" if $_dev_mode && $_debug_breakpoints;
         %$result = %{$_loaded_breakpoints{$real_path}};
     }
 
@@ -1020,15 +1022,15 @@ sub _get_loaded_breakpoints_by_real_path
     if (my $substituted_file_descriptor = $_evals_to_templates_map{$real_path})
     {
         my ($template_path, $lines_map) = @$substituted_file_descriptor{qw/path lines_map/};
-        _report "Found template file %s", $template_path if $_dev_mode;
+        _report "Found template file %s", $template_path if $_dev_mode && $_debug_breakpoints;
         if (my $template_breakpoints = $_loaded_breakpoints{$template_path})
         {
-            _report "Found template breakpoints" if $_dev_mode;
+            _report "Found template breakpoints" if $_dev_mode && $_debug_breakpoints;
             foreach my $line (keys %$template_breakpoints)
             {
                 if (my $mapped_line = $lines_map->{$line})
                 {
-                    _report "Got mapped breakpoint %s => %s", $line, $mapped_line if $_dev_mode;
+                    _report "Got mapped breakpoint %s => %s", $line, $mapped_line if $_dev_mode && $_debug_breakpoints;
                     $result->{$mapped_line} //= $template_breakpoints->{$line};
                 }
             }
@@ -1127,7 +1129,7 @@ sub _set_breakpoint
     };
 
     _report 'Setting breakpoint to %s, real line %s, %s', $breakpoint_descriptor->{path}, $real_line,
-        $perl_source_lines->[$real_line] if $_dev_mode;
+        $perl_source_lines->[$real_line] if $_dev_mode && $_debug_breakpoints;
 
     if (!defined $perl_source_lines->[$real_line] || $perl_source_lines->[$real_line] == 0)
     {
@@ -1485,6 +1487,8 @@ sub sub_handler
     my $stack_frame = undef;
 
     my $old_db_single = $DB::single;
+    my $wantarray = wantarray;
+
     if (!$_internal_process)
     {
         $_internal_process = 1;
@@ -1494,7 +1498,7 @@ sub sub_handler
 
         $DB::single = STEP_CONTINUE;
 
-        $stack_frame = _enter_frame( $old_db_single );
+        $stack_frame = _enter_frame( $old_db_single, $wantarray );
 
         if ($current_package && $current_package eq 'DB')
         {
@@ -1509,8 +1513,6 @@ sub sub_handler
 
     my $stack_pointer = $#$_stack_frames;
 
-    my $wantarray = wantarray;
-
     if ($DB::single == STEP_OVER)
     {
         _report "Disabling step in in subcalls, will restore %s\n",
@@ -1543,9 +1545,9 @@ sub sub_handler
             $DB::single = $old_db_single;
         }
 
-        return $DB::ret = undef;
+        $DB::ret = undef; # return value
     }
-    if ($wantarray)
+    elsif ($wantarray)
     {
         no strict 'refs';
         my @result = &$DB::sub;
@@ -1558,7 +1560,7 @@ sub sub_handler
         {
             $DB::single = $old_db_single;
         }
-        return @DB::ret = @result;
+        @DB::ret = @result; # return value
     }
     else
     {
@@ -1573,7 +1575,7 @@ sub sub_handler
         {
             $DB::single = $old_db_single;
         }
-        return $DB::ret = $result;
+        $DB::ret = $result; # return value
     }
 }
 
